/**
 * The <code>Node</code> class is an implementation
 * of a node of the Option PDAG.  Each node contains
 * a Key object, references to successor nodes, and 
 * references to nodes with the same time value.
 * <p>
 *
 * @author  Somesh Jha
 * @author  Jeff Stephenson
 */


public class Node 
{

    /**
     * The (Node) reference to the previous node with this time value 
     */
    public Node prev;

    /**
     * The (Node) reference to the next node with this time value 
     */
    public Node next;

    /**
     * The (KeyInterface) data value
     */
    public KeyInterface  key;

    /**
     * The (int) number of successors of this node.
     * Currently set to 2 (i.e. the binomial model).
     */
    static final int NO_OF_SUCC = 2;

    /**
     * The (Node) array of successors 
     */
    Node succ[] = new Node[NO_OF_SUCC];


    /**
     * Constructs a Node object, and sets the initial 
     * key data value.  The prev and next references are 
     * set to null.
     *
     * @param   k			the (KeyInterface) data value
     */
    public Node(KeyInterface k) 
    {
        key  = k;
        prev = null;
        next = null;
    }

    /**
     * Searches the link-list starting at this node
     * for the specified key
     *
     * @param   k			the (KeyInterface) data value to search for
     */
    public Node Search(KeyInterface k) 
    {
        Node x = this;
        while (x != null) 
        {
            if (x.key.isEqual(k))  
            {
                return(x);
            }
            else 
            {
                x = x.next;
            }
        }
        //didn't find the key
        return(x);
    }


    /**
     * Inserts the given node after this node.
     *
     * @param   x			the (Node) to insert
     */
    public void InsertAfter(Node x) 
    {
        Node temp = this.next;
        this.next = x;
        x.prev = this;
        x.next = temp;
    }

    /**
     * Inserts the given node before this node.
     *
     * @param   x			the (Node) to insert
     */
    public void InsertBefore(Node x) 
    {
        Node temp = this.prev;
        this.prev = x;
        x.next = this;
        x.prev = temp;
    }


    /**
     * Inserts the given key after this node
     * if it is not already in the list
     *
     * @param   k		the KeyInterface to the key to insert
     */
    public Node InsertElement(KeyInterface k) 
    {
        Node x = Search(k);

        if (x == null) 
        {
            x = new Node(k);
            InsertAfter(x);
        }
        return(x);
    }

    /**
     * Removes this node from the linked list
     *
     */
    public void Delete() 
    {
        Node prev1 = this.prev;
        Node next1 = this.next;

        System.out.print("Node -- Delete: prev and next ");
        prev1.key.printKey();
        next1.key.printKey();

        if (prev1 != null)
        {
            prev1.next = next1;
        }
        
        if (next1 != null) 
        {
            next1.prev = prev1;
        }
    }
    
    /**
     * Prints this node object
     *
     */
    public void printNode() 
    {
        Node x=this;
        while (x != null) 
        {
            x.key.printKey();
            x = x.next;
        }
    }
        
}
